# Poulpe Den : The recommended setup for Poulpe

## What is a Poulpe Den

A Poulpe Den is a directory with its contents structured in a standard way that
simplifies the usage of Poulpe for benchmarking. It contains and manages the
basic building blocks of Poulpe:

- binary environments, that represent code to be benchmarked,
- data environments, that represent the data on which to run the code,
- the available benchmarks, that represent the operations to benchmark,
- the results, that contain the data gathered during benchmark runs.

Its operations revolve a lot around "suites" that gather domain-specific
definitions of setup information for most of the building block listed above.

# Setting up a Poulpe Den

> NOTE: if you have a proxy, you should pass it through an environment variable
> because the suites are going to clone the default repositories to test.

The Poulpe source directory contains a `bin/setup-poulpe-den` script. To setup
your Poulpe Den, simply run:

  ```shell
  $ python3.7-and-up ./bin/setup-poulpe-den TARGET_DEV_PATH
  $ cd TARGET_DEV_PATH
  # Activate the virtual environment through the custom activate
  # script to get completion
  $ . bin/activate
  $ poulpe
  [The help of poulpe and its commands]
  ```

## Using your Poulpe Den

### Data environments

The first and most manual thing about Poulpe is setting up data environments,
since they are specific to what you are actually trying to benchmark.
(See their section in [concept.md](concept.md))

In the following example, we will look at the suite for benchmarking Mercurial.
To do this, we need at least one Mercurial repository to run our tests on.

  ```shell
  # inside the poulpe den folder, with the virtualenv active
  $ mkdir data-envs/my-repo
  # `main` is the default path for the reference of the repo to test inside
  # the data-env
  $ hg clone /path/to/my-repo data-envs/my-repo/main
  $ ./suites/hg/data-env-setup/setup-full.py data-envs/my-repo
  [...]

  # You can check that the den sees the new environment
  $ poulpe den-check
  [...]
  data environments:     [OKAY]
    available: 1
  [...]
  ```

Look at the help and source of the suite you are using to customize it to your
needs.

### Generating tasks

Poulpe has tools to help you generate tasks quickly.

  ```shell
  $ poulpe den-generate-tasks <spec> > current.poulpe-schedule
  ```

The spec describes the parameters that resolve to a binary environment.

For example "I want to benchmark Mercurial, at version 6.8, with the Rust
extensions and Python 3.11" is:
`hg:MERCURIAL_VERSION=6.8:MERCURIAL_FLAVOR=rust:PYTHON_VERSION=python3.11`

Run `poulpe den-interactive-bin-env-spec` to get guided through the creation
of a spec, or use `poulpe den-list-bin-env-setups` to show all available parameters.

You also can list the available benchmarks and information about them.

  ```shell
  $ poulpe den-list-benchmarks
  hg.command.cat
  hg.command.commit
  [...]
  ```

Note: `den-list-benchmarks` has flags to show more information like the benchmark
variants and where they're stored, etc.

### Running tasks

Given a schedule file (current.poulpe-schedule by default), for each task,
Poulpe will automatically generate (or re-use) the binary environment, then
run the benchmark and save its results.

  ```shell
  $ poulpe den-execute-tasks
  [...] # Some output about bin-env creation if need be
  [1/N] running <bin-env-spec> <data-env> <benchmark-spec>
  [2/N] running <bin-env-spec> <data-env> <benchmark-spec>
  [...]
  ```

### Comparing results

Poulpe has a couple of lower-level tools to help you make sense of the
results. A better high-level interface should come soon.

  ```shell
  $ poulpe result-search results | poulpe result-compare
  ### data-env-vars.name            = my-data-env
    # benchmark.name                = suite.my.benchmark
    # bin-env-vars.my.var           = value
    # bin-env-vars.my.other-var     = other-value
    # benchmark.variants.variant1   = 1000
    # benchmark.variants.variant2   = no
  <hash1>: 0.500159
  <hash2>: 0.467283   (-6.57%)
  <hash3>: 0.500900
  ```

You can also pass `-d/--dimension` to `result-compare` to change which dimension
is used to compare the results, like "benchmark.name" or "bin-env-vars.my.var".

For more complex filtering of results, you can use the following:

  ```shell
  $ poulpe result-search results --filter <filter> | poulpe result-compare
  ```

### Beyond the Poulpe Den

If the Den setup becomes too constrained or impractical for some of your
use-cases or development needs, keep in mind that each building block of Poulpe
is designed to also be handled independently from the Den structure and
connected in custom ways when needed. See [concept.md](concept.md) for details.
