import os
from poulpe.utils import split_unescape
import toml

BIN_ENV_FILE = "bin-env.poulpe"
SHELL_FILE = "bin-env.shell"
DATA_ENV_FILE = "data-env.poulpe"


def bin_env_file(env_path):
    """given the path of env, return the file of description file"""
    return os.path.join(env_path, BIN_ENV_FILE)


def bin_env_script(bin_env_path):
    """given the path of env, return the file of description file"""
    return os.path.join(bin_env_path, SHELL_FILE)


def data_env_file(data_env_path):
    """given the path of env, return the file of description file"""
    return os.path.join(data_env_path, DATA_ENV_FILE)


def get_data(path):
    """Read a descript file at <path>"""
    try:
        with open(path) as f:
            return toml.load(f)
    except FileNotFoundError:
        return None


def write_data(path, data):
    """write description data at <path>"""
    directory = os.path.dirname(path)
    basename = os.path.basename(path)
    tmp_path = os.path.join(directory, f".{basename}.tmp")
    with open(tmp_path, "w") as f:
        toml.dump(data, f)
    os.replace(tmp_path, path)


def show(data, write_fn, indent=""):
    """call to display data"""
    if not isinstance(data, dict):
        write_fn(f"{indent}{data}")
        return
    for k, v in sorted(data.items()):
        if isinstance(v, dict):
            write_fn(f"{indent}{k}:")
            show(v, write_fn, indent + "  ")
        elif isinstance(v, list):
            write_fn(f"{indent}{k}:")
            # This doesn't work for lists with non-primitive elements
            for i in v:
                show(i, write_fn, indent + "- ")
        else:
            write_fn(f"{indent}{k} = {v}")


def get_one_value(data, key, default=None):
    key_path = split_unescape(key, ".")
    sub = data
    for k in key_path:
        sub = sub.get(k)
        if sub is None:
            break
    if sub is None:
        sub = default
    return sub


def set_one_value(data, key, value):
    key_path = split_unescape(key, ".")
    sub = data
    for k in key_path[:-1]:
        val = sub.setdefault(k, {})
        sub = val
    sub[key_path[-1]] = value


def del_one_value(data, key):
    key_path = split_unescape(key, ".")
    sub = data
    for k in key_path[:-1]:
        sub = sub.get(k)
        if sub is None:
            break
    if sub is not None:
        sub.pop(key_path[-1], None)


def set_value(path, key, value):
    data = get_data(path)
    if data is None:
        data = {}

    set_one_value(data, key, value)

    write_data(path, data)
    return 0


def del_value(path, key):
    data = get_data(path)
    if data is None:
        data = {}

    key_path = split_unescape(key, ".")
    sub = data
    for k in key_path[:-1]:
        sub = sub.setdefault(k, {})
    sub.pop(key_path[-1], None)

    write_data(path, data)
    return 0
