import subprocess
import json

from . import (
    basics,
)


def get_binary_env(bin_env_path):
    bin_env_desc = basics.bin_env_file(bin_env_path)
    data = basics.get_data(bin_env_desc)
    return BinaryEnvironment(bin_env_path, data)


class BinaryEnvironment:
    def __init__(self, path, data):
        self._path = path
        self._data = data

    @property
    def path(self):
        return self._path

    @property
    def bin_env_vars(self):
        return self._data["bin-env-vars"].copy()


class ParametersError(RuntimeError):
    pass


class BinEnvSetup:
    ParameterT = dict[str, dict]

    def __init__(self, path):
        self._path = path

    def get_parameters(self) -> ParameterT:
        cmd = [self._path, "--describe-parameters"]
        process = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        process.wait()
        desc, err = process.communicate()
        if process.returncode != 0:
            raise ParametersError(err)
        return json.loads(desc)["parameters"]
