import re


def parse_filters(filter_definitions=None):
    """parse command line parsers definition

    return a list of [(key, value)] tuple where:
    - key is a tuple with the "path" to the value we match
    - value is the value to match. if the value is prefixed with `re:`, it will
      match a regular exception.
    """
    if filter_definitions is None:
        return []
    filters = []
    for fd in filter_definitions:
        key, value = fd.split("=", 1)

        key = key.split(".")
        if value.startswith("re:"):
            value = value[3:]
        else:
            value = re.escape(value)
        value = re.compile(value)
        filters.append((key, value))
    return filters


def apply_filters(filters, data):
    """yield items in `results` that match all filters"""
    for key, value in filters:
        treck = list(key[::-1])
        current = data
        while treck and current is not None:
            k = treck.pop()
            current = current.get(k)
        if current is None:
            return False
        current = str(current)
        if not value.match(current):
            return False
    else:
        return True


def build_pattern(patterns=None):
    re_pieces = []
    if patterns is None:
        re_pieces.append(".*")
    else:
        for p in patterns:
            if p.startswith("re:"):
                re_pieces.append(p[3:])
            else:
                re_pieces.append("^" + re.escape(p))
    return re.compile("|".join(re_pieces))
