import poulpe
import sys

from collections import defaultdict
from pathlib import Path


def load_results_from_dir(result_dir):
    """return a list of result data from a directory"""
    results = []
    for item in result_dir.iterdir():
        if item.suffix == ".pbr":
            results.append(poulpe.get_data(item))
    return results


def group_results(results):
    """dispatch a list of result into consistent subgroup

    for now lets keep it simple:
        benchmark -> results
    """
    benchmarks = defaultdict(list)
    for r in results:
        b_name = r["benchmark"]["name"]
        benchmarks[b_name].append(r)
    for res in benchmarks.values():
        res.sort(key=result_sort_key)

    return benchmarks


def result_sort_key(result):
    return parse_version(result["bin-env-vars"]["hg"]["version"])


def parse_version(version):
    """return a sortable key for a given hg_version"""
    if "+" in version:
        v_parts, rank_s = version.split(sep="+", maxsplit=1)
    else:
        v_parts = version
        rank_s = "0"

    rc = 100
    pieces = []
    if "rc" in v_parts:
        v_parts, rc = v_parts.split(sep="rc", maxsplit=1)

    v = v_parts.split(sep=".")
    pieces.extend(int(x) for x in v)
    while len(pieces) < 3:
        pieces.append(0)
    pieces.append(int(rc))

    rank_s = rank_s.split(sep=".")[0]
    rank_s = rank_s.split(sep="-")[0]

    if "hg" in rank_s:
        rank_s = rank_s.split(sep="hg", maxsplit=1)[1]
    pieces.append(int(rank_s))
    return tuple(pieces)


if __name__ == "__main__":
    import json

    target_dir = Path(sys.argv[1])
    results = load_results_from_dir(target_dir)
    dispatched = group_results(results)
    json.dump(dispatched, sys.stdout)
