import click

import glob
import json
import os
from .basics import get_data
from . import filter


def apply_filter(filters, results):
    """yield items in `results` that match all filters"""
    for path in results:
        data = get_data(path)
        for key, value in filters:
            treck = list(key[::-1])
            current = data
            while treck and current is not None:
                k = treck.pop()
                current = current.get(k)
            if current is None:
                break
            current = str(current)
            if not value.match(current):
                break
        else:
            yield path


def find_results(result_paths):
    results = []
    for r in result_paths:
        if os.path.isdir(r):
            glob_pattern = os.path.join(r, "*.pbr")
            files = glob.glob(glob_pattern)
            results.extend(files)
        elif os.path.isfile(r):
            results.append(r)
    return results


def filter_results(filters, results):
    matching_results = []
    for path in results:
        data = get_data(path)
        if filter.apply_filters(filters, data):
            matching_results.append(path)
    return matching_results


def content_as_json(results):
    """display a list of results as json"""
    first = True
    click.echo("[")
    for path in results:
        if not first:
            click.echo(",")
        first = False
        data = get_data(path)
        click.echo(json.dumps(data))
    click.echo("]")
