use std::path::PathBuf;

use indexmap::IndexMap;

use crate::runner::{
    mercurial_perf::MercurialPerfBenchmark,
    simple_command::SimpleCommandBenchmark,
};

#[derive(Debug, PartialEq)]
pub enum BenchmarkMethod {
    SimpleCommand(SimpleCommandBenchmark),
    MercurialPerfExtension(MercurialPerfBenchmark),
    Unknown,
}

#[derive(PartialEq)]
pub struct Benchmark {
    pub suite_name: String,
    pub environment: IndexMap<String, String>,
    pub path: PathBuf,
    pub name: String,
    pub method: BenchmarkMethod,
    /// Contents of the entire file
    pub data: toml::Value,
}

impl Eq for Benchmark {}

impl Benchmark {
    pub fn qualified_name(&self) -> String {
        format!("{}:{}", self.suite_name, self.name)
    }
}
