use std::path::PathBuf;

use indexmap::IndexMap;
use serde::Deserialize;

pub struct BinaryEnvironment {
    pub root: PathBuf,
    /// TODO record spec?
    pub spec: String,
    pub is_ready: bool,
    /// Corresponds to the bin-env.poulpe contents
    pub metadata: toml::Value,
}

impl BinaryEnvironment {
    pub fn name(&self) -> &str {
        self.root
            .file_name()
            .expect("no filename")
            .to_str()
            .expect("non utf8 filename")
    }
}

#[derive(Deserialize)]
pub struct Setup {
    #[serde(skip)]
    pub path: PathBuf,
    pub parameters: IndexMap<String, SetupParameter>,
}

#[derive(Debug, Deserialize)]
pub struct SetupParameter {
    pub default: Option<String>,
    pub doc: Option<String>,
    #[serde(rename(deserialize = "type"))]
    pub typ: String,
    pub choices: Option<IndexMap<String, String>>,
    #[serde(rename(deserialize = "arg-mode"))]
    pub arg_mode: ArgMode,
}

#[derive(Debug, Deserialize)]
pub enum ArgMode {
    CLI,
    ENV,
}
