use std::{collections::HashMap, path::PathBuf};

use serde::Deserialize;
use toml::Value;

#[derive(Deserialize)]
pub struct BenchmarkResult {
    pub path: PathBuf,
    pub contents: BenchmarkResultContents,
    /// Corresponds to the full deserialized results file
    pub data: toml::Value,
}

#[derive(Deserialize)]
#[serde(rename_all = "kebab-case")]
pub struct BenchmarkResultContents {
    pub run: RunSection,
    pub benchmark: BenchmarkSection,
    pub data_env_vars: HashMap<String, Value>,
    pub bin_env_vars: HashMap<String, Value>,
    pub result: ResultSection,
}

#[derive(Deserialize)]
pub struct RunSection {
    // TODO use jiffy
    pub timestamp: f64,
    pub duration: f64,
}

#[derive(Deserialize)]
pub struct BenchmarkSection {
    pub name: String,
}

#[derive(Deserialize)]
#[serde(rename_all = "kebab-case")]
pub struct ResultSection {
    pub time: TimeSection,
    /// Mercurial perf extension
    pub sys_time: Option<TimeSection>,
    /// Mercurial perf extension
    pub comb_time: Option<TimeSection>,
    /// Mercurial perf extension
    pub run_count: Option<usize>,
}

#[derive(Deserialize)]
#[serde(rename_all = "kebab-case")]
pub struct TimeSection {
    pub median: f64,
    pub mean: f64,
    pub standard_deviation: Option<f64>,
    /// Mercurial perf extension does not provide a min
    pub min: Option<f64>,
    pub max: f64,
}
