use indexmap::IndexMap;
use serde::Deserialize;

#[derive(Debug, PartialEq, Deserialize)]
pub struct MercurialPerfCommandVariantCase {
    #[serde(default)]
    pub default: bool,
    pub args: Option<Vec<String>>,
    pub cwd: Option<String>,
    pub setup_script: Option<String>,
    // TODO revive this
    pub constraints: Option<toml::Value>,
}

// TODO maybe the format should be flattened? This is useless nesting so far
#[derive(Debug, PartialEq, Deserialize)]
pub struct MercurialPerfCommandVariants {
    pub dimensions:
        IndexMap<String, IndexMap<String, MercurialPerfCommandVariantCase>>,
}

impl MercurialPerfCommandVariants {
    pub fn default_key(&self, dimension: &str) -> Option<&str> {
        for (name, info) in self.dimensions.get(dimension)? {
            if info.default {
                return Some(name);
            }
        }
        None
    }
}

#[derive(Debug, PartialEq, Deserialize)]
pub struct MercurialPerfBenchmark {
    pub command: String,
    pub setup_script: Option<String>,
    pub args: Option<Vec<String>>,
    pub variants: Option<MercurialPerfCommandVariants>,
}
