use indexmap::IndexMap;
use serde::Deserialize;

#[derive(Debug, PartialEq, Deserialize)]
pub struct SimpleCommandVariantCase {
    #[serde(default)]
    pub environment: IndexMap<String, String>,
    #[serde(default)]
    pub default: bool,
    pub extend_command: Option<String>,
    pub cwd: Option<String>,
    pub acceptable_return_codes: Option<Vec<u8>>,
    // TODO revive this
    pub constraints: Option<toml::Value>,
}

#[derive(Debug, PartialEq, Deserialize)]
pub struct SimpleCommandVariants {
    pub dimensions:
        IndexMap<String, IndexMap<String, SimpleCommandVariantCase>>,
}

impl SimpleCommandVariants {
    pub fn default_key(&self, dimension: &str) -> Option<&str> {
        for (name, info) in self.dimensions.get(dimension)? {
            if info.default {
                return Some(name);
            }
        }
        None
    }
}

#[derive(Debug, PartialEq, Deserialize)]
pub struct SimpleCommandBenchmark {
    pub command: String,
    pub gather_info: Option<String>,
    pub prepare_run: Option<String>,
    pub cleanup_run: Option<String>,

    pub variants: Option<SimpleCommandVariants>,
    #[serde(default)]
    pub accept_failure: bool,
}
