use std::path::Path;

use crate::{benchmark::Benchmark, utils::escape_string};

pub fn build_scheduler_run_instruction<'a>(
    benchmark: &Benchmark,
    bin_parameters: impl IntoIterator<Item = (&'a str, &'a str)>,
    bench_parameters: impl IntoIterator<Item = (&'a str, &'a str)>,
    data_env_path: impl AsRef<Path>,
) -> String {
    let bin_parameters = bin_parameters.into_iter().fold(
        String::new(),
        |mut acc, (field, value)| {
            acc.push(':');
            acc.push_str(&escape_string(field, ":="));
            acc.push('=');
            acc.push_str(&escape_string(value, ":="));
            acc
        },
    );
    let bench_parameters = bench_parameters.into_iter().fold(
        String::new(),
        |mut acc, (field, value)| {
            acc.push(':');
            acc.push_str(&escape_string(field, ":="));
            acc.push('=');
            acc.push_str(&escape_string(value, ":="));
            acc
        },
    );
    format!(
        "RUN {}{} {} {}/{}{}\n",
        benchmark.suite_name,
        bin_parameters,
        data_env_path.as_ref().display(),
        benchmark.suite_name,
        benchmark.name,
        bench_parameters,
    )
}
