use crate::{app::DOCS_DIR, Dashboard};
use egui::{CentralPanel, Context, RichText, ScrollArea, SidePanel};
use egui_commonmark::CommonMarkViewer;

const VIEWER_MAX_WIDTH: f32 = 800.0;

pub fn render_docs_tab(dashboard: &mut Dashboard, ctx: &Context) {
    ctx.style_mut(|style| {
        // Restore text wrapping here because we're displaying the docs.
        style.wrap_mode = Some(egui::TextWrapMode::Wrap);
    });

    SidePanel::left("docs-left-panel").show(ctx, |ui| {
        for file in DOCS_DIR.files() {
            if let Some(path) = file.path().to_str() {
                let link = if dashboard.current_doc_page == path {
                    ui.link(RichText::new(path).strong())
                } else {
                    ui.link(path)
                };

                if link.clicked() {
                    dashboard.current_doc_page = path.to_string();
                }
            };
        }
    });
    CentralPanel::default().show(ctx, |ui| {
        let markdown_viewer = CommonMarkViewer::new();
        ScrollArea::vertical()
            .id_salt(format!("doc-page: {}", &dashboard.current_doc_page))
            .max_width(VIEWER_MAX_WIDTH)
            .show(ui, |ui| {
                for (hook, clicked) in dashboard.commonmark_cache.link_hooks() {
                    if *clicked {
                        dashboard.current_doc_page = hook.to_string();
                        // No need to check more in the (impossible?)
                        // case where multiple clicks are registered
                        // within the same frame
                        break;
                    }
                }
                ui.heading(&dashboard.current_doc_page);
                markdown_viewer.show(
                    ui,
                    &mut dashboard.commonmark_cache,
                    DOCS_DIR
                        .get_file(&dashboard.current_doc_page)
                        .expect("doc page should exist")
                        .contents_utf8()
                        .expect("doc page should be valid UTF8"),
                );
            });
    });
}
