use crate::{app::DashboardMode, Dashboard, IS_WEB, WARNING_COLOR};
use egui::{CentralPanel, Context, SidePanel};
use egui_taffy::{taffy::prelude::*, TuiBuilderLogic};

pub fn render_results_tab(dashboard: &mut Dashboard, ctx: &Context) {
    ctx.style_mut(|style| {
        // Disable text wrapping
        style.wrap_mode = Some(egui::TextWrapMode::Truncate);
    });
    SidePanel::left("results-left-panel").show(ctx, |ui| {
        if let Some(_den) = &dashboard.den {
            ui.label("TODO");
        } else {
            if !IS_WEB {
                ui.label("Use the top bar to point the dashboard to a den");
            } else {
                ui.colored_label(
                    WARNING_COLOR,
                    "Remote loading of a den is not available yet",
                );
            }
            if ui.button("Open the docs").clicked() {
                dashboard.mode = DashboardMode::Docs;
            }
        }
    });
    ctx.style_mut(|style| {
        // Disable text wrapping
        style.wrap_mode = Some(egui::TextWrapMode::Extend);
    });
    CentralPanel::default().show(ctx, |ui| {
        let Some(_den) = &dashboard.den else {
            dashboard.no_den_message(ui);
            return;
        };
        egui_taffy::tui(ui, ui.id().with("results-grid"))
            .reserve_available_space()
            .style(Style {
                // display: Display::Grid,
                ..Default::default()
            })
            .show(|tui| {
                tui.heading("TODO");
            })
    });
}
