use crate::Dashboard;
use egui::{CentralPanel, Context, SidePanel};
use egui_taffy::{
    taffy::{prelude::*, Overflow, Point},
    TuiBuilderLogic,
};

// TODO make proportional to font size?
pub const FORM_TARGET_WIDTH: f32 = 350.0;

pub fn render_run_tab(dashboard: &mut Dashboard, ctx: &Context) {
    let available_side_space = ctx.screen_rect().width() - FORM_TARGET_WIDTH;
    let default_width = ctx.screen_rect().width() * 0.5;

    SidePanel::right("run-right-panel")
        .default_width(default_width)
        .min_width(300.0)
        .max_width(available_side_space)
        .show(ctx, |ui| {
            if dashboard.den.is_some() {
                dashboard.benchmark_form.render_schedule(ui);
            } else {
                dashboard.no_den_message(ui);
            }
        });
    CentralPanel::default().show(ctx, |ui| {
        if let Some(den) = dashboard.den.as_ref() {
            egui_taffy::tui(ui, ui.id().with("run-form"))
                .reserve_available_space()
                .style(Style {
                    flex_direction: FlexDirection::Row,
                    justify_content: Some(AlignContent::Center),
                    align_items: Some(AlignItems::Center),
                    min_size: Size {
                        width: percent(1.0),
                        height: percent(1.0),
                    },
                    ..Default::default()
                })
                .show(|tui| {
                    tui.style(Style {
                        flex_direction: FlexDirection::Column,
                        overflow: Point {
                            x: Overflow::default(),
                            y: Overflow::Scroll,
                        },
                        max_size: Size {
                            width: length(FORM_TARGET_WIDTH),
                            height: auto(),
                        },
                        gap: length(10.0),
                        padding: length(10.0),
                        ..Default::default()
                    })
                    .add_with_border(|tui| {
                        dashboard.benchmark_form.render_form(tui, den);
                    });
                });
        }
    });
}
