open module benchmark
import "package://pkg.pkl-lang.org/pkl-pantry/pkl.toml@1.0.0#/toml.pkl"
// First basic and raw translation of the .toml structure in pkl

/// The way this benchmark is expected to be run. It will also define most of the
/// other value we expect to find to configure this benchmark.
///
/// The current list of supported method is :
///
/// :simple-command: benchmarking the run of a single shell command.
/// :mercurial-perf-extension: using Mercurial perf extensions to run benchmarks
///
/// This should be automatically be set by an extending template
method: Method

/// identifier of the benchmark. This will be stored in result.
name: String

/// A section that allow to set environment variable to be used when
/// running the command. This is useful to replace some of the command
/// argument with. Each sub-key is a an environment variable to replace.
///
/// Value supports the `DATA-VARS:` notation if necessary
environment: Environment

/// When true, the benchmark is run a copy of the data-environment. Any
/// alteration will be destroyed at the end of the iteration.
///
/// (XXX its more complicated than that with btrfs snapshot…)
copy_data_env: Boolean = true


/// The directory to run the benchmark in
///
/// Value can be substituded for data-env variable using the
/// `"DATA-VARS:path.to.variable.in.the.data.env"` syntax.
///
/// For example `"DATA-VARS:mercurial.status.large.all"`
///
/// The default should be the root of the data-env.
///
/// This can be changed by variants (see next section).
cwd: String?

/// Theses variables control when this benchmark can be run.
constraints: Constraints?



///////////////////////////////////////////////////////////////////////////////
/// rendering as toml /////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

// render as toml

output {
    value = as_v0
    renderer = new toml.Renderer {}
}

hidden as_v0: Dynamic = new {
    ["meta"] = new Dynamic {
        ["format"] = "0"
        ["name"] = name
        ["method"] = method
    }
    ["run"] = new  Dynamic {
        ["copy-data-env"] = copy_data_env
    }
    when (!environment.isEmpty) {
        ["environment"] = environment
    }
}


///////////////////////////////////////////////////////////////////////////////
/// base classes and utilities ////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

typealias Method = "simple-command"|"mercurial-perf-extension"


/// override of environment variable
typealias Environment = Mapping<String, String>

class Constraints{
    // XXX could be less generic now, and probably need more generic later
    // no longer used by the scheduler anyway right now
    graph: GraphConstraint = new {}
    /// The range of revision compatible with this changes.
    /// (as a revset)
    compatible_revision_range: String?

    hidden fixed as_v0: Dynamic = new{
        when (compatible_revision_range != null) {
            ["compatible-revision-range"] = compatible_revision_range
        }
        when (graph.has_values != null) {
            ["graph"] = graph.as_v0
        }
    }


}

class GraphConstraint {
  /// Target repository must have at least that many visible revision for this
  /// benchmark to run.
  visible_revision_count: NumericalConstraint?
  hidden has_values = visible_revision_count != null
  hidden fixed as_v0: Dynamic = new {
        when (visible_revision_count != null){
            ["visible-revision-count"] = visible_revision_count.as_v0
        }
  }
}

/// set at least one of min or max
class NumericalConstraint {
    min: Int?
    max: Int?
    hidden fixed as_v0: Mapping<String, Int> = new {
        when (min != null) {
            ["min"] = min
        } else {
            ["max"] = max
        }
    }
}

