extends "../../../_common/benchmarks/benchmark.pkl"

method = "mercurial-perf-extension"

/// The perf-command to use.
command: String
/// Arguments to pass to the command (a list of string).
args: Listing<String>
/// a shell script to run before running the perf-command.
setup_script: String?
/// Variants allow for slight variation of a benchmark. For example, they allow to change the
/// input data, of to turn some feature on and off.
///
/// Variants are organised within independant "dimensions".
///
/// Each dimension has a set of cases, each value can modify the benchmark. Each
/// dimension must have a default value.
variants: Mapping<String, PerfVariant>

///////////////////////////////////////////////////////////////////////////////
/// rendering as toml /////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
as_v0 = (super.as_v0) {
    ["hg-perf-ext"] = new Dynamic {
        ["command"] = command
        when (!args.isEmpty) {
            ["args"] = args
        }
        when (setup_script != null) {
            ["setup-script"] = setup_script
        }
        when (module.cwd != null) {
            ["cwd"] = module.cwd
        }
        when (module.constraints != null) {
            ["constraints"] = module.constraints.as_v0
        }
        when (!variants.isEmpty) {
            ["variants"] = new Mapping{
                ["dimensions"] = new Mapping {
                    for (k, v in variants) {
                        [k] = v.as_v0
                    }
                }
            }
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
/// base classes and utilities ////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

class PerfVariant {
    /// The variant that should be used by default
    default_key: String((s) -> cases.containsKey(s))
    cases: Mapping<String, PerfVariantCase>

    /// render as "v0" format
    hidden fixed as_v0: Dynamic = new {
        // XXX assert there is one default key
        for (k, v in cases) {
            [k] = (v) {
                default = k == default_key
            }.as_v0
        }
    }
}

class PerfVariantCase {
    /// Exactly one variant must have it set to `true` per dimension.
    ///
    /// When set and no variant have been explicitly selected for this dimension, this
    /// variant will be used.
    default: Boolean = false
    /// A list of args to add to the command
    args: Listing<String>
    /// overwrite the global `hg-perf-ext.cwd` variable. Behave the same.
    cwd: String?
    /// overwrite the global `hg-perf-ext.setup-script` variable. Behave the same.
    setup_script: String?

    // additional constraint on when this variant can be used
    constraints: Constraints?

    /// render as "v0" format
    hidden fixed as_v0: Dynamic = new {
        when (default) {
            ["default"] = default
        }
        when (!args.isEmpty) {
            ["args"] = args
        }
        when (setup_script != null) {
            ["setup-script"] = setup_script
        }
        when (cwd != null) {
            ["cwd"] = cwd
        }
        when (constraints != null) {
            ["constraints"] = constraints.as_v0
        }
    }
}
