// first basic campaign template using pkl
// requires more documentation to be used by people
// first put to use in "revlog-campaing.pkl" in the meantime

/// A "Renderer" class to generate a shell script that generate a campaign
///
/// The intermediate shell script is currently a simple solution to the lack of
/// easy "argument" to the pkl script.
class DenGenerateTaskCaller extends ValueRenderer {
    function renderDocument(task_specs: Listing<TaskSpec>): String = (
        """
        #!/bin/bash
        for VERSION in "$@"; do
        echo version: $VERSION >&2;
        """ + render_one_version(task_specs) + """
        done;\n
        """
    )
    function render_one_version(task_specs: Listing<TaskSpec>): String = (
        task_specs
        .toList()
        .flatMap(
            (ts) -> ts.dgt_calls.toList()
        ).join(";\n") + ";\n"
    )
}


typealias Flavor = "default"|"pure"|"no-rust"|"rust"|"rhg"
typealias Re2 = "google-re2"

class VersionSpec {
    fixed suite: String = "hg"
    // side step the feeding issue for now
    fixed mercurial_version: String(!isEmpty) = "$VERSION"
    python_version: String(!isEmpty)?
    mercurial_repo_url: String(!isEmpty)?
    mercurial_flavor: Flavor = "default"
    mercurial_py_re2: Re2?
    evolve_version: String(!isEmpty)?
    evolve_repo_url: String(!isEmpty)?
    is_rust: Boolean = mercurial_flavor == "rust" || mercurial_flavor == "rhg"

    fixed as_arg: String = new Listing {
        suite
        "MERCURIAL_VERSION=\(mercurial_version)"
        when (mercurial_flavor != "default") {
            "MERCURIAL_FLAVOR=\(mercurial_flavor)"
        }
        when (evolve_version != null) {
            "EVOLVE_VERSION=\(evolve_version)"
        }
        when (python_version != null) {
            "PYTHON_VERSION=\(python_version)"
        }
        when (mercurial_py_re2 != null) {
            "MERCURIAL_PY_RE2=\(mercurial_py_re2)"
        }
        when (mercurial_repo_url != null) {
            "MERCURIAL_REPO_URL=\(mercurial_repo_url)"
        }
        when (evolve_repo_url != null) {
            "MERCURIAL_REPO_URL=\(evolve_repo_url)"
        }
    }.toList().join(":")
}

class DataEnvFilter {
    key: String
    // should use native RE type
    value: String
    fixed as_args: Listing<String> = new {
        "--data-env-filter"
        "\(key)='\(value)'"
    }
}

class BenchmarkNameFilter {
    // should use native RE type
    value: String
    fixed as_args: Listing<String> = new {
        "--benchmark-name-filter"
        "'\(value)'"
    }
}

class BenchmarkVariantFilter {
    key: String
    // should use native RE type
    value: String
    fixed as_args: Listing<String> = new {
        "--benchmark-variant-filter"
        "\(key)='\(value)'"
    }
}

const local function args_iter(specs: Listing): Listing<Listing<String>> = new {
    when (specs.isEmpty) {
        new Listing{""}
    } else {
        for (s in specs) {
            s.as_args
        }
    }
}

class TaskSpec {
    versions: Listing<VersionSpec>(!isEmpty, isDistinct) = new { new {} }
    data_env_specs: Listing<DataEnvFilter>(isDistinct)
    benchmark_name_specs: Listing<BenchmarkNameFilter>(isDistinct)
    benchmark_variant_specs: Listing<BenchmarkVariantFilter>(isDistinct)
    fixed dgt_calls: Listing<String> = new {
        for (name_arg in args_iter(benchmark_name_specs)) {
            for (ver in versions) {
                for (data_arg in args_iter(data_env_specs)) {
                    for (variant_arg in args_iter(benchmark_variant_specs)) {
                        new Listing {
                            "poulpe"
                            "den-generate-tasks"
                            ver.as_arg
                            name_arg.join(" ")
                            args_iter(data_env_specs).toList().flatten().join(" ")
                            args_iter(benchmark_variant_specs).toList().flatten().join(" ")
                        }.join(" ")
                    }
                }
            }
        }
    }
}


const NO_RUST: VersionSpec = new {
    mercurial_flavor="no-rust"
}
const RUST: VersionSpec = new {
    mercurial_flavor="rust"
}
const RHG: VersionSpec = new {
    mercurial_flavor="rhg"
}

tasks: Listing<TaskSpec>


output {
    value = tasks
    renderer = new DenGenerateTaskCaller{}
}
