#!/bin/bash
#
# Simple script to record graph information about the data-env's repo
set -Eeuo pipefail

if [ $# -ne 1 ]; then
    echo >&2 "USAGE: setup-01-graph.sh DATA-ENV-PATH"
    echo >&2 "  DATA-ENV-PATH: the path to that data-env"
    exit 32
fi

data_env_directory=$1
desc="${data_env_directory}/data-env.poulpe"

repository_path="`poulpe env-desc get "$desc" bench-input-vars.mercurial.main-repo-path`"

repo="$data_env_directory/$repository_path"

token="special-token-sdljlajslgjasljdlkasjlgjsjga-match-me"

nb_revs=`(hg log --template "${token}\n" -R $repo | grep $token || true) | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.visible-revision-count $nb_revs

draft_revs=`(hg log --template "${token}\n" -R $repo --rev 'draft()'| grep $token || true) | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.draft-visible-revision-count $draft_revs

secret_revs=`(hg log --template "${token}\n" -R $repo --rev 'draft()'| grep $token || true) | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.secret-visible-revision-count $secret_revs

obsolete_revs=`(hg log --template "${token}\n" -R $repo --rev 'obsolete()'| grep $token || true) | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.obsolete-visible-revision-count $obsolete_revs

non_public_revs=`(hg log --template "${token}\n" -R $repo --rev 'not public()'| grep $token || true) | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.non-public-visible-revision-count $non_public_revs

hidden_revs=`(hg log --template "${token}\n" -R $repo --rev 'hidden()' --hidden| grep $token || true) | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.hidden-revision-count $hidden_revs

obsolete_hidden_revs=`(hg log --template "${token}\n" -R $repo --rev 'obsolete()' --hidden | grep $token || true) | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.obsolete-hidden-revision-count $obsolete_hidden_revs

obsmarker_count=`hg debugobsolete -R $repo | wc -l`
poulpe env-desc set --type int "$desc" data-env-vars.mercurial.repo.graph.obsmarker-count $obsmarker_count
