amends "lib/simple-command.pkl"

name = "swh.git-loader.from-disk.postgres"

// we do not modify the loaded project
copy_data_env = false

local db_name = "poulpe-swh-git-loader"
local batch_item_size = 100
local batch_bytes_size = 52428800
local loader_conf_path = "loader-conf.yml"
local storage_conf_path = "storage-conf.yml"
local pids_file = "rpc-serve-pid"
local pack_file_limit = (20.gib).toUnit("b").value
local lift_limit= "pack_size_bytes=\(pack_file_limit)"

environment {
    ["TARGET_REPO"] = "DATA-VARS:swh.swh-loader-git.target-repo-path"
}

command = """
swh -l ERROR loader -C "$POULPE_TMP_DIR/\(loader_conf_path)" run git "$TARGET_REPO" \(lift_limit) > /dev/null
"""

// TODO Use a separate pgcluster for this to make sure we don't do anything stupid
// TODO This requires CREATE DATABASE privileges anyway and no authentication,
// TODO so no huge risk right now
gather_info = """
dropdb \(db_name) || true
createdb \(db_name)
postgres_server_version=$(psql --no-psqlrc \(db_name) -t -c 'SHOW server_version;')
postgres_client_version=$(psql --no-psqlrc --version)
dropdb \(db_name)
cat > "$POULPE_TMP_DIR/gathered_info.json" << EOF
{
    "postgres": {
        "server_version": "$postgres_server_version",
        "client_version": "$postgres_client_version"
    }
}
EOF
"""

prepare_run = """
dropdb \(db_name) 2>/dev/null || true
createdb \(db_name)
swh -l ERROR db init-admin -d dbname=\(db_name) storage >/dev/null 2>&1
swh -l ERROR db init -d dbname=\(db_name) storage >/dev/null 2>&1

cat > "$POULPE_TMP_DIR/\(loader_conf_path)" << EOF
storage:
  cls: pipeline
  steps:
  - cls: buffer
    min_batch_size:
      content: \(batch_item_size)
      content_bytes: \(batch_bytes_size)
      directory: \(batch_item_size)
      directory_entries: \(batch_item_size * 5)
      extid: \(batch_item_size)
      release: \(batch_item_size)
      release_bytes: \(batch_bytes_size)
      revision: \(batch_item_size)
      revision_bytes: \(batch_bytes_size)
      revision_parents: \(batch_item_size * 2)
  - cls: filter
  - cls: retry
  - cls: remote
    url: http://127.0.0.1:5002
EOF

mkdir "$POULPE_TMP_DIR/objstorage"

# XXX run that one with python3.11 not pypy

cat > "$POULPE_TMP_DIR/\(storage_conf_path)" << EOF
storage:
  cls: postgresql
  db: dbname=\(db_name)
  objstorage:
    cls: pathslicing
    root: $POULPE_TMP_DIR/objstorage
    slicing: "0:2/2:4/4:6"
EOF

# TODO process groups?

swh -l ERROR storage -C "$POULPE_TMP_DIR/\(storage_conf_path)" rpc-serve >/dev/null 2>&1 &
echo "$!" > "$POULPE_TMP_DIR/\(pids_file)"

# in the preload case, load the data so that the benchmark has nothing left to do
if [ PRELOAD == "all" ]; then
    swh -l ERROR loader -C "$POULPE_TMP_DIR/\(loader_conf_path)" run git "$TARGET_REPO" \(lift_limit)> /dev/null
fi
sync
"""

cleanup_run = """
cat "$POULPE_TMP_DIR/\(pids_file)" | xargs kill
dropdb \(db_name) 2>/dev/null || true
rm -rf "$POULPE_TMP_DIR"
mkdir --parents "$POULPE_TMP_DIR"
"""

variants {
    ["preload"] {
        default_key = "none"
        cases {
            for (preload in List("none", "all")) {
                [preload] {
                    environment {
                        ["PRELOAD"] = preload
                    }
                }
            }
        }
    }
}
