Test the utility that manage bin-env creation
---------------------------------------------

  $ . $TESTDIR/testlib/base-test-setup.sh
  $ python -m venv $TESTTMP/
  $ . $TESTTMP/bin/activate
# pip is apparently unable to have multiple instance installing from the same
# directory at the same time, so we do a local copy first ☹
  $ cp -aR  $TESTDIR/.. poulpe-dir
  $ pip install ./poulpe-dir/ --quiet
  $ rm -rf poulpe-dir

Check basic invocation

  $ poulpe bin-env-util --help
  Usage: bin-env-util [OPTIONS] COMMAND [ARGS]...
  
    Tool to manipulate binary environments
  
  Options:
    --help  Show this message and exit.
  
  Commands:
    create-empty  Create a new empty bin-env at a given path
    del           Delete a specific bin-env variable.
    get           Show a specific bin-env variable.
    mark-ready    Mark a bin-env as ready.
    set           Set the value of a specific bin-env variable.
    setup-one     Create a new bin-env with a setup script.
    show          Show all data we have about this environment.


Check repository creation

simple creation

  $ poulpe bin-env-util create-empty test-create
  $ ls -1 test-create
  bin-env.poulpe
  $ poulpe bin-env-util show test-create
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = manual
  ready = False

Running on an existing repo abort and does not destroy it

  $ poulpe bin-env-util create-empty test-create
  This is already a bin-env, aborting
  [4]
  $ poulpe bin-env-util show test-create
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = manual
  ready = False

Creating a repository on an existing dir

  $ mkdir test-existing-dir
  $ poulpe bin-env-util create-empty test-existing-dir
  $ poulpe bin-env-util show test-existing-dir
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = manual
  ready = False

Check manual setup
------------------

Check changing variable manually

  $ python3 -m venv test-existing-dir
  $ cd test-existing-dir
  $ bin/pip3 install --quiet black==18.6b4
  $ poulpe bin-env-util set . python.version "`bin/python --version | egrep -o '\S+$'`"
  $ poulpe bin-env-util set . black.version "`bin/black --version | egrep -o '\S+$'`"
  $ poulpe bin-env-util set . black.install-method pip
  $ poulpe bin-env-util show .
  bin-env-vars:
    black:
      install-method = pip
      version = 18.6b4
    python:
      version = 3.* (glob)
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = manual
  ready = False

Marking it ready when it is not

  $ poulpe bin-env-util mark-ready .
  cannot find an executable file at: "./bin-env.shell"
  [2]
  $ poulpe bin-env-util show .
  bin-env-vars:
    black:
      install-method = pip
      version = 18.6b4
    python:
      version = 3.* (glob)
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = manual
  ready = False

Manually create an activate script

  $ cat > bin-env.shell << EOF
  > #!/bin/bash
  > BIN_ENV_PATH="\`dirname \"\$0\"\`"
  > . "\$BIN_ENV_PATH"/bin/activate
  > "\$@"
  > EOF
  $ poulpe bin-env-util mark-ready .
  cannot find an executable file at: "./bin-env.shell"
  [2]
  $ chmod +x bin-env.shell

Marking it ready when it is

  $ poulpe bin-env-util mark-ready .
  $ poulpe bin-env-util show .
  bin-env-vars:
    black:
      install-method = pip
      version = 18.6b4
    python:
      version = 3.* (glob)
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = manual
  ready = True

Basic test of the result

  $ ./bin-env.shell echo babar
  babar
  $ ./bin-env.shell black --version
  black, version 18.6b4
  $ cd ..

Test scripted setup
-------------------

Create setup scripts

  $ cat > setup-error.sh << EOF
  > #!/bin/bash
  > exit 2
  > EOF
  $ chmod +x setup-error.sh

  $ grep -v 'chmod' $TESTDIR/test-data/setup-black.sh > setup-black-bad-activate.sh
  $ chmod +x setup-black-bad-activate.sh

Test a missing script

  $ poulpe bin-env-util setup-one env-missing setup-missing.sh
  no executable setup script at: "setup-missing.sh"
  [6]

Test a setup error

  $ poulpe bin-env-util setup-one env-error setup-error.sh
  script returned with status 2: $TESTTMP/setup-error.sh
  [7]

Test an invalid activate

  $ poulpe bin-env-util setup-one env-bad-activate setup-black-bad-activate.sh
  $TESTTMP/setup-black-bad-activate.sh: line 5: BLACK_VERSION: unbound variable
  script returned with status 1: $TESTTMP/setup-black-bad-activate.sh
  [7]
  $ rm -rf env-bad-activate
  $ BLACK_VERSION="18.6b4" poulpe bin-env-util setup-one  env-bad-activate setup-black-bad-activate.sh
  cannot find an executable file at: "env-bad-activate/bin-env.shell"
  [2]

Test a succesfull script

  $ BLACK_VERSION="18.6b4" poulpe bin-env-util setup-one  env-black-18.6b4 \
  > $TESTDIR/test-data/setup-black.sh
  $ BLACK_VERSION="18.6b1" poulpe bin-env-util setup-one  env-black-18.6b1 \
  > $TESTDIR/test-data/setup-black.sh

  $ poulpe bin-env-util show env-black-18.6b1
  bin-env-vars:
    black:
      install-method = pip
      version = 18.6b1
    python:
      version = 3.* (glob)
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = script
  ready = True
  $ env-black-18.6b1/bin-env.shell black --version
  black, version 18.6b1

  $ poulpe bin-env-util show env-black-18.6b4
  bin-env-vars:
    black:
      install-method = pip
      version = 18.6b4
    python:
      version = 3.* (glob)
  poulpe-environment:
    environment-type = binary
    format-version = 0
    setup-method = script
  ready = True
  $ env-black-18.6b4/bin-env.shell black --version
  black, version 18.6b4
